@@ -1,6 +1,6 @@
 validate_X <- function(X) {
   if(inherits(X, "matrix") & !is.numeric(X)) {
-    stop(paste("The feature matrix X must numeric. GRF does not", 
+    stop(paste("The feature matrix X must be numeric. GRF does not",
          "currently support non-numeric features. If factor variables",
          "are required, we recommend one of the following: Either",
          "represent the factor with a 1-vs-all expansion,",
@@ -11,6 +11,32 @@ validate_X <- function(X) {
   if (inherits(X, "Matrix") & !(inherits(X, "dgCMatrix"))) {
       stop("Currently only sparse data of class 'dgCMatrix' is supported.")
   }
+
+  if (any(is.na(X))){
+    stop("The feature matrix X contains at least one NA.")
+  }
+}
+
+validate_observations <- function(V, X) {
+  if (is.matrix(V) && ncol(V) == 1) {
+    V = as.vector(V)
+  } else if (!is.vector(V)){
+    stop(paste( "Observations (W, Y, or Z) must be vectors."))
+  }
+
+  if (!is.numeric(V) && !is.logical(V)) {
+    stop(paste( "Observations (W, Y, or Z) must be numeric. GRF does not ",
+                "currently support non-numeric observations."))
+  }
+
+  if (any(is.na(V))) {
+    stop("The vector of observations (W, Y, or Z) contains at least one NA.")
+  }
+
+  if (length(V) != nrow(X)) {
+    stop("length of observation (W, Y, or Z) does not equal nrow(X).")
+  }
+  V
 }
 
 validate_mtry <- function(mtry, X) {
@@ -77,14 +103,17 @@ validate_seed <- function(seed) {
 }
 
 validate_clusters <- function(clusters, X) {
-  if (is.null(clusters)) {
-    clusters <- vector(mode="numeric", length=0)
-  } else if (length(clusters) == 0) {
-    clusters <- vector(mode="numeric", length=0)
-  } else if (!is.vector(clusters) | !all(clusters == floor(clusters))) {
-    stop("Clusters must be a vector of integers.")
+  if (is.null(clusters) || length(clusters) == 0) {
+    return (vector(mode="numeric", length=0))
+  }
+  if (mode(clusters) != "numeric") {
+    stop("Clusters must be able to be coerced to a numeric vector.")
+  }
+  clusters <- as.numeric(clusters)
+  if (!all(clusters == floor(clusters))) {
+    stop("Clusters vector cannot contain floating point values.")
   } else if (length(clusters) != nrow(X)) {
-    stop("Clusters has incorrect length.")
+    stop("Clusters vector has incorrect length.")
   } else {
     # convert to integers between 0 and n clusters
     clusters <- as.numeric(as.factor(clusters)) - 1
@@ -92,18 +121,18 @@ validate_clusters <- function(clusters, X) {
   clusters
 }
 
-validate_samples_per_cluster <- function(samples_per_cluster, clusters) {
+validate_samples_per_cluster <- function(samples.per.cluster, clusters) {
   if (is.null(clusters) || length(clusters) == 0) {
     return(0)
   }
   cluster_size_counts <- table(clusters)
   min_size <- unname(cluster_size_counts[order(cluster_size_counts)][1])
-  if (is.null(samples_per_cluster)) {
-    samples_per_cluster <- min_size
-  } else if (samples_per_cluster <= 0) {
-    stop("samples_per_cluster must be positive")
+  if (is.null(samples.per.cluster)) {
+    samples.per.cluster <- min_size
+  } else if (samples.per.cluster <= 0) {
+    stop("samples.per.cluster must be positive")
   }
-  samples_per_cluster
+  samples.per.cluster
 }
 
 validate_honesty_fraction <- function(honesty.fraction, honesty) {
@@ -123,7 +152,15 @@ validate_honesty_fraction <- function(honesty.fraction, honesty) {
   }
 }
 
-validate_ll_vars <- function(linear.correction.variables, num.cols){
+validate_boost_error_reduction <- function(boost.error.reduction) {
+  if(boost.error.reduction <0 || boost.error.reduction >1 ) {
+    stop("boost.error.reduction must be between 0 and 1")
+  }
+  boost.error.reduction
+
+}
+
+validate_ll_vars <- function(linear.correction.variables, num.cols) {
   if (is.null(linear.correction.variables)) {
     linear.correction.variables = 1:num.cols
   }
@@ -137,7 +174,7 @@ validate_ll_vars <- function(linear.correction.variables, num.cols){
   linear.correction.variables
 }
 
-validate_ll_lambda <- function(lambda){
+validate_ll_lambda <- function(lambda) {
   if (lambda < 0) {
     stop("Lambda cannot be negative.")
   } else if (!is.numeric(lambda) | length(lambda) > 1) {
@@ -146,7 +183,7 @@ validate_ll_lambda <- function(lambda){
   lambda
 }
 
-validate_ll_path <- function(lambda.path){
+validate_ll_path <- function(lambda.path) {
   if (is.null(lambda.path)) {
     lambda.path = c(0, 0.001, 0.01, 0.05, 0.1, 0.3, 0.5, 0.7, 1, 10)
   } else if (min(lambda.path)<0) {
@@ -157,6 +194,24 @@ validate_ll_path <- function(lambda.path){
   lambda.path
 }
 
+validate_newdata <- function(newdata, X) {
+  if (ncol(newdata) != ncol(X)) {
+    stop("newdata must have the same number of columns as the training matrix.")
+  }
+  validate_X(newdata)
+}
+
+validate_sample_weights = function(sample.weights, X) {
+    if(!is.null(sample.weights)) {
+        if(length(sample.weights) != nrow(X)) {
+            stop("sample.weights has incorrect length")
+        }
+        if(any(sample.weights < 0)) {
+            stop("sample.weights must be nonnegative")
+        }
+    }
+}
+
 coerce_honesty_fraction <- function(honesty.fraction) {
   if(is.null(honesty.fraction)) {
     return(0)
@@ -164,14 +219,17 @@ coerce_honesty_fraction <- function(honesty.fraction) {
   honesty.fraction
 }
 
-create_data_matrices <- function(X, ...) {
-  default.data <- matrix(nrow=0, ncol=0);    
+#' @importFrom Matrix Matrix cBind
+#' @importFrom methods new
+create_data_matrices <- function(X, ..., sample.weights=NULL) {
+  default.data <- matrix(nrow=0, ncol=0);
   sparse.data <- new("dgCMatrix", Dim = c(0L, 0L))
 
   if (inherits(X, "dgCMatrix") && ncol(X) > 1) {
-    sparse.data <- cbind(X, ...)
+    sparse.data <- cbind(X, ..., sample.weights)
   } else {
-    default.data <- as.matrix(cbind(X, ...))
+    X <- as.matrix(X)
+    default.data <- as.matrix(cbind(X, ..., sample.weights))
   }
 
   list(default = default.data, sparse = sparse.data)
