@@ -28,27 +28,40 @@ ForestPredictor::ForestPredictor(uint num_threads,
 }
 
 ForestPredictor::ForestPredictor(uint num_threads,
-                                 uint ci_group_size,
                                  std::shared_ptr<OptimizedPredictionStrategy> strategy) :
     tree_traverser(num_threads) {
   this->prediction_collector = std::shared_ptr<PredictionCollector>(
-      new OptimizedPredictionCollector(strategy, ci_group_size));
+      new OptimizedPredictionCollector(strategy));
 }
 
-std::vector<Prediction> ForestPredictor::predict(const Forest& forest, Data* data) const {
-  return predict(forest, data, false);
+
+std::vector<Prediction> ForestPredictor::predict(const Forest& forest,
+                                                 Data* train_data,
+                                                 Data* data,
+                                                 bool estimate_variance) const {
+  return predict(forest, train_data, data, estimate_variance, false);
 }
 
-std::vector<Prediction> ForestPredictor::predict_oob(const Forest& forest, Data* data) const {
-  return predict(forest, data, true);
+std::vector<Prediction> ForestPredictor::predict_oob(const Forest& forest,
+                                                     Data* data,
+                                                     bool estimate_variance) const {
+  return predict(forest, data, data, estimate_variance, true);
 }
 
 std::vector<Prediction> ForestPredictor::predict(const Forest& forest,
+                                                 Data* train_data,
                                                  Data* data,
+                                                 bool estimate_variance,
                                                  bool oob_prediction) const {
+  if (estimate_variance && forest.get_ci_group_size() <= 1) {
+    throw std::runtime_error("To estimate variance during prediction, the forest must"
+       " be trained with ci_group_size greater than 1.");
+  }
+
   std::vector<std::vector<size_t>> leaf_nodes_by_tree = tree_traverser.get_leaf_nodes(forest, data, oob_prediction);
   std::vector<std::vector<bool>> trees_by_sample = tree_traverser.get_valid_trees_by_sample(forest, data, oob_prediction);
 
-  return prediction_collector->collect_predictions(
-      forest, data, leaf_nodes_by_tree, trees_by_sample, oob_prediction);
+  return prediction_collector->collect_predictions(forest, train_data, data,
+      leaf_nodes_by_tree, trees_by_sample,
+      estimate_variance, oob_prediction);
 }
