@@ -24,39 +24,29 @@
 #include "commons/utility.h"
 #include "ForestTrainer.h"
 
-ForestTrainer::ForestTrainer(const std::unordered_map<size_t, size_t>& observables,
-                             std::shared_ptr<RelabelingStrategy> relabeling_strategy,
+ForestTrainer::ForestTrainer(std::shared_ptr<RelabelingStrategy> relabeling_strategy,
                              std::shared_ptr<SplittingRuleFactory> splitting_rule_factory,
                              std::shared_ptr<OptimizedPredictionStrategy> prediction_strategy) :
-    observables(observables),
-    tree_trainer(observables,
-                 relabeling_strategy,
+    tree_trainer(relabeling_strategy,
                  splitting_rule_factory,
                  prediction_strategy) {}
 
-const Forest ForestTrainer::train(Data* data,
+const Forest ForestTrainer::train(const Data* data,
                                   const ForestOptions& options) const {
   size_t num_samples = data->get_num_rows();
   uint num_trees = options.get_num_trees();
 
-  // Ensure that the sample fraction is not too small.
+  // Ensure that the sample fraction is not too small and honesty fraction is not too extreme.
+  const TreeOptions& tree_options = options.get_tree_options();
+  bool honesty = tree_options.get_honesty();
+  double honesty_fraction = tree_options.get_honesty_fraction();
   if ((size_t) num_samples * options.get_sample_fraction() < 1) {
     throw std::runtime_error("The sample fraction is too small, as no observations will be sampled.");
+  } else if (honesty && ((size_t) num_samples * options.get_sample_fraction() * honesty_fraction < 1
+             || (size_t) num_samples * options.get_sample_fraction() * (1-honesty_fraction) < 1)) {
+    throw std::runtime_error("The honesty fraction is too close to 1 or 0, as no observations will be sampled.");
   }
 
-  size_t num_types = observables.size();
-  std::vector<std::vector<double>> observations_by_type(num_types);
-  for (auto it : observables) {
-    size_t type = it.first;
-    size_t index = it.second;
-
-    observations_by_type[type].resize(num_samples);
-    for (size_t row = 0; row < data->get_num_rows(); ++row) {
-      observations_by_type[type][row] = data->get(row, index);
-    }
-  }
-  Observations observations(observations_by_type, data->get_num_rows());
-
   uint num_groups = (uint) num_trees / options.get_ci_group_size();
 
   std::vector<uint> thread_ranges;
@@ -78,7 +68,6 @@ const Forest ForestTrainer::train(Data* data,
                                  start_index,
                                  num_trees_batch,
                                  data,
-                                 observations,
                                  options));
   }
 
@@ -87,16 +76,15 @@ const Forest ForestTrainer::train(Data* data,
     trees.insert(trees.end(), thread_trees.begin(), thread_trees.end());
   }
 
-  return Forest::create(trees, data, observables);
+  return Forest::create(trees, options, data);
 }
 
 std::vector<std::shared_ptr<Tree>> ForestTrainer::train_batch(
     size_t start,
     size_t num_trees,
-    Data* data,
-    const Observations& observations,
+    const Data* data,
     const ForestOptions& options) const {
-  uint ci_group_size = options.get_ci_group_size();
+  size_t ci_group_size = options.get_ci_group_size();
 
   std::mt19937_64 random_number_generator(options.get_random_seed() + start);
   std::uniform_int_distribution<uint> udist;
@@ -113,27 +101,25 @@ std::vector<std::shared_ptr<Tree>> ForestTrainer::train_batch(
     RandomSampler sampler(tree_seed, options.get_sampling_options());
 
     if (ci_group_size == 1) {
-      std::shared_ptr<Tree> tree = train_tree(data, observations, sampler, options);
+      std::shared_ptr<Tree> tree = train_tree(data, sampler, options);
       trees.push_back(tree);
     } else {
-      std::vector<std::shared_ptr<Tree>> group = train_ci_group(data, observations, sampler, options);
+      std::vector<std::shared_ptr<Tree>> group = train_ci_group(data, sampler, options);
       trees.insert(trees.end(), group.begin(), group.end());
     }
   }
   return trees;
 }
 
-std::shared_ptr<Tree> ForestTrainer::train_tree(Data* data,
-                                                const Observations& observations,
+std::shared_ptr<Tree> ForestTrainer::train_tree(const Data* data,
                                                 RandomSampler& sampler,
                                                 const ForestOptions& options) const {
   std::vector<size_t> clusters;
   sampler.sample_clusters(data->get_num_rows(), options.get_sample_fraction(), clusters);
-  return tree_trainer.train(data, observations, sampler, clusters, options.get_tree_options());
+  return tree_trainer.train(data, sampler, clusters, options.get_tree_options());
 }
 
-std::vector<std::shared_ptr<Tree>> ForestTrainer::train_ci_group(Data* data,
-                                                                 const Observations& observations,
+std::vector<std::shared_ptr<Tree>> ForestTrainer::train_ci_group(const Data* data,
                                                                  RandomSampler& sampler,
                                                                  const ForestOptions& options) const {
   std::vector<std::shared_ptr<Tree>> trees;
@@ -146,7 +132,7 @@ std::vector<std::shared_ptr<Tree>> ForestTrainer::train_ci_group(Data* data,
     std::vector<size_t> cluster_subsample;
     sampler.subsample(clusters, sample_fraction * 2, cluster_subsample);
 
-    std::shared_ptr<Tree> tree = tree_trainer.train(data, observations,
+    std::shared_ptr<Tree> tree = tree_trainer.train(data,
         sampler, cluster_subsample, options.get_tree_options());
     trees.push_back(tree);
   }
