@@ -17,10 +17,9 @@
 
 #include <cmath>
 #include <iostream>
-#include <string>
 #include <vector>
 
-#include "commons/Observations.h"
+#include "commons/Data.h"
 #include "commons/utility.h"
 #include "prediction/InstrumentalPredictionStrategy.h"
 
@@ -46,7 +45,7 @@ std::vector<double> InstrumentalPredictionStrategy::predict(const std::vector<do
 std::vector<double> InstrumentalPredictionStrategy::compute_variance(
     const std::vector<double>& average,
     const PredictionValues& leaf_values,
-    uint ci_group_size) {
+    size_t ci_group_size) {
 
   double instrument_effect_numerator = average.at(OUTCOME_INSTRUMENT)
      - average.at(OUTCOME) * average.at(INSTRUMENT);
@@ -152,7 +151,7 @@ size_t InstrumentalPredictionStrategy::prediction_value_length() {
 
 PredictionValues InstrumentalPredictionStrategy::precompute_prediction_values(
     const std::vector<std::vector<size_t>>& leaf_samples,
-    const Observations& observations) {
+    const Data* data) {
   size_t num_leaves = leaf_samples.size();
 
   std::vector<std::vector<double>> values(num_leaves);
@@ -172,37 +171,45 @@ PredictionValues InstrumentalPredictionStrategy::precompute_prediction_values(
     double sum_YZ = 0;
     double sum_WZ = 0;
 
+    double sum_weight = 0.0;
     for (auto& sample : leaf_samples[i]) {
-      sum_Y += observations.get(Observations::OUTCOME, sample);
-      sum_W += observations.get(Observations::TREATMENT, sample);
-      sum_Z += observations.get(Observations::INSTRUMENT, sample);
-      sum_YZ += observations.get(Observations::OUTCOME, sample) * observations.get(Observations::INSTRUMENT, sample);
-      sum_WZ += observations.get(Observations::TREATMENT, sample) * observations.get(Observations::INSTRUMENT, sample);
+      auto weight = data->get_weight(sample);
+      sum_Y +=  weight * data->get_outcome(sample);
+      sum_W +=  weight * data->get_treatment(sample);
+      sum_Z +=  weight * data->get_instrument(sample);
+      sum_YZ += weight * data->get_outcome(sample) * data->get_instrument(sample);
+      sum_WZ += weight * data->get_treatment(sample) * data->get_instrument(sample);
+      sum_weight += weight;
     }
 
-    value[OUTCOME] = sum_Y / leaf_size;
-    value[TREATMENT] = sum_W / leaf_size;
-    value[INSTRUMENT] = sum_Z / leaf_size;
-    value[OUTCOME_INSTRUMENT] = sum_YZ / leaf_size;
-    value[TREATMENT_INSTRUMENT] = sum_WZ / leaf_size;
+    // if total weight is very small, treat the leaf as empty
+    if (std::abs(sum_weight) <= 1e-16) {
+      continue;
+    }
+
+    value[OUTCOME] = sum_Y / sum_weight;
+    value[TREATMENT] = sum_W / sum_weight;
+    value[INSTRUMENT] = sum_Z / sum_weight;
+    value[OUTCOME_INSTRUMENT] = sum_YZ / sum_weight;
+    value[TREATMENT_INSTRUMENT] = sum_WZ / sum_weight;
   }
   
-  return PredictionValues(values, num_leaves, NUM_TYPES);
+  return PredictionValues(values, NUM_TYPES);
 }
 
-std::vector<double> InstrumentalPredictionStrategy::compute_debiased_error(
+std::vector<std::pair<double, double>> InstrumentalPredictionStrategy::compute_error(
     size_t sample,
     const std::vector<double>& average,
     const PredictionValues& leaf_values,
-    const Observations& observations) {
+    const Data* data) {
 
   double instrument_effect_numerator = average.at(OUTCOME_INSTRUMENT) - average.at(OUTCOME) * average.at(INSTRUMENT);
   double first_stage_numerator = average.at(TREATMENT_INSTRUMENT) - average.at(TREATMENT) * average.at(INSTRUMENT);
   double treatment_effect_estimate = instrument_effect_numerator / first_stage_numerator;
 
-  double outcome = observations.get(Observations::OUTCOME, sample);
-  double treatment = observations.get(Observations::TREATMENT, sample);
-  double instrument = observations.get(Observations::INSTRUMENT, sample);
+  double outcome = data->get_outcome(sample);
+  double treatment = data->get_treatment(sample);
+  double instrument = data->get_instrument(sample);
 
   // To justify the squared residual below as an error criterion in the case of CATE estimation
   // with an unconfounded treatment assignment, see Nie and Wager (2017).
@@ -221,7 +228,7 @@ std::vector<double> InstrumentalPredictionStrategy::compute_debiased_error(
   // If the treatment effect estimate is due to less than 5 trees, do not attempt to estimate error,
   // as this quantity is unstable due to non-linearities.
   if (num_trees <= 5) {
-    return { NAN };
+    return { std::make_pair<double, double>(NAN, NAN) };
   }
 
   // Compute 'leave one tree out' treatment effect estimates, and use them get a jackknife estimate of the excess error.
@@ -243,6 +250,13 @@ std::vector<double> InstrumentalPredictionStrategy::compute_debiased_error(
     error_bias += (residual_loto - residual) * (residual_loto - residual);
   }
 
-  error_bias *= ((double)(num_trees - 1)) / (num_trees);
-  return { error_raw - error_bias };
+
+  error_bias *= ((double) (num_trees - 1)) / num_trees;
+
+  double debiased_error = error_raw - error_bias;
+
+  auto output = std::make_pair(debiased_error, error_bias);
+  return {output};
+
 }
+
