@@ -38,20 +38,21 @@ public:
   size_t prediction_value_length();
   PredictionValues precompute_prediction_values(
       const std::vector<std::vector<size_t>>& leaf_samples,
-      const Observations& observations);
+      const Data* data);
 
   size_t prediction_length();
+
   std::vector<double> predict(const std::vector<double>& average);
 
   std::vector<double> compute_variance(const std::vector<double>& average,
                           const PredictionValues& leaf_values,
-                          uint ci_group_size);
+                          size_t ci_group_size);
 
-  std::vector<double> compute_debiased_error(
+  std::vector<std::pair<double, double>>  compute_error(
       size_t sample,
       const std::vector<double>& average,
       const PredictionValues& leaf_values,
-      const Observations& observations);
+      const Data* data);
 
 private:
   ObjectiveBayesDebiaser bayes_debiaser;
