@@ -21,23 +21,16 @@
 #include <iostream>
 #include "Eigen/Dense"
 #include "commons/utility.h"
-#include "commons/Observations.h"
+#include "commons/Data.h"
 #include "prediction/LocalLinearPredictionStrategy.h"
 
-
-const size_t LocalLinearPredictionStrategy::OUTCOME = 0;
-
 size_t LocalLinearPredictionStrategy::prediction_length() {
   return lambdas.size();
 }
 
-LocalLinearPredictionStrategy::LocalLinearPredictionStrategy(const Data* original_data,
-                                                             const Data* test_data,
-                                                             std::vector<double> lambdas,
+LocalLinearPredictionStrategy::LocalLinearPredictionStrategy(std::vector<double> lambdas,
                                                              bool weight_penalty,
                                                              std::vector<size_t> linear_correction_variables):
-        original_data(original_data),
-        test_data(test_data),
         lambdas(lambdas),
         weight_penalty(weight_penalty),
         linear_correction_variables(linear_correction_variables){
@@ -46,7 +39,8 @@ LocalLinearPredictionStrategy::LocalLinearPredictionStrategy(const Data* origina
 std::vector<double> LocalLinearPredictionStrategy::predict(
     size_t sampleID,
     const std::unordered_map<size_t, double>& weights_by_sampleID,
-    const Observations& observations) {
+    const Data* train_data,
+    const Data* data) {
   size_t num_variables = linear_correction_variables.size();
   size_t num_nonzero_weights = weights_by_sampleID.size();
 
@@ -68,10 +62,10 @@ std::vector<double> LocalLinearPredictionStrategy::predict(
   for (size_t i = 0; i < num_nonzero_weights; ++i) {
     for (size_t j = 0; j < num_variables; ++j){
       size_t current_predictor = linear_correction_variables[j];
-      X(i,j+1) = original_data->get(indices[i],current_predictor)
-                 - test_data->get(sampleID, current_predictor);
+      X(i,j+1) = train_data->get(indices[i],current_predictor)
+                 - data->get(sampleID, current_predictor);
     }
-    Y(i) = observations.get(Observations::OUTCOME, indices[i]);
+    Y(i) = train_data->get_outcome(indices[i]);
     X(i, 0) = 1;
   }
 
@@ -100,8 +94,8 @@ std::vector<double> LocalLinearPredictionStrategy::predict(
       }
     }
 
-    Eigen::MatrixXd preds = M.ldlt().solve(X.transpose()*weights_vec.asDiagonal()*Y);
-    predictions[i] =  preds(0);
+    Eigen::MatrixXd local_coefficients = M.ldlt().solve(X.transpose()*weights_vec.asDiagonal()*Y);
+    predictions[i] =  local_coefficients(0);
   }
 
   return predictions;
@@ -111,15 +105,16 @@ std::vector<double> LocalLinearPredictionStrategy::compute_variance(
     size_t sampleID,
     std::vector<std::vector<size_t>> samples_by_tree,
     std::unordered_map<size_t, double> weights_by_sampleID,
-    const Observations& observations,
-    uint ci_group_size) {
+    const Data* train_data,
+    const Data* data,
+    size_t ci_group_size) {
 
   double lambda = lambdas[0];
 
   size_t num_variables = linear_correction_variables.size();
   size_t num_nonzero_weights = weights_by_sampleID.size();
 
-  std::vector<size_t> sample_index_map(observations.get_num_samples());
+  std::vector<size_t> sample_index_map(train_data->get_num_rows());
   std::vector<size_t> indices(num_nonzero_weights);
 
   Eigen::MatrixXd weights_vec = Eigen::VectorXd::Zero(num_nonzero_weights);
@@ -141,10 +136,10 @@ std::vector<double> LocalLinearPredictionStrategy::compute_variance(
     X(i, 0) = 1;
     for (size_t j = 0; j < num_variables; ++j){
       size_t current_predictor = linear_correction_variables[j];
-      X(i,j+1) = original_data->get(indices[i],current_predictor)
-                 - test_data->get(sampleID, current_predictor);
+      X(i,j+1) = train_data->get(indices[i],current_predictor)
+                 - data->get(sampleID, current_predictor);
     }
-    Y(i) = observations.get(Observations::OUTCOME, indices[i]);
+    Y(i) = train_data->get_outcome(indices[i]);
   }
 
   // find ridge regression predictions
