@@ -22,7 +22,7 @@
 #include <vector>
 
 #include "commons/globals.h"
-#include "commons/Observations.h"
+#include "commons/Data.h"
 #include "prediction/Prediction.h"
 #include "prediction/PredictionValues.h"
 
@@ -66,7 +66,7 @@ public:
   virtual std::vector<double> compute_variance(
       const std::vector<double>& average_prediction_values,
       const PredictionValues& leaf_prediction_values,
-      uint ci_group_size) = 0;
+      size_t ci_group_size) = 0;
 
  /**
   * The number of types of precomputed prediction values. For regression
@@ -84,13 +84,25 @@ public:
   */
   virtual PredictionValues precompute_prediction_values(
       const std::vector<std::vector<size_t>>& leaf_samples,
-      const Observations& observations) = 0;
+      const Data* data) = 0;
 
-  virtual std::vector<double> compute_debiased_error(
+ /**
+  * Computes a pair of estimates for (out-of-bag debiased error, monte-carlo error) for a single sample.
+  * The 'debiased error' is the expected error for a forest containing an infinite number of trees.
+  * The 'monte-carlo error' is the error inherent in the algorithm randomization, i.e. a measure of how
+  * different predictions from two forests grown on the same data can be.
+  *
+  * sample: index of the observation
+  * leaf_values: collected prediction values from all leaves across forests
+  * observations: depending on the forest type, this may contain output, treatment
+  *     and/or instrument values. These are used to compute an estimate of the
+  *     error given leaf_values.
+  */
+  virtual std::vector<std::pair<double, double>> compute_error(
       size_t sample,
       const std::vector<double>& average,
       const PredictionValues& leaf_values,
-      const Observations& observations) = 0;
+      const Data* data) = 0;
 };
 
 
