@@ -24,13 +24,16 @@
 class Prediction {
 public:
   Prediction(const std::vector<double>& predictions);
+
   Prediction(const std::vector<double>& predictions,
              const std::vector<double>& variance_estimates,
-             const std::vector<double>& error_estimates);
+             const std::vector<double>& error_estimates,
+             const std::vector<double>& excess_error_estimates);
 
   const std::vector<double>& get_predictions() const;
   const std::vector<double>& get_variance_estimates() const;
   const std::vector<double>& get_error_estimates() const;
+  const std::vector<double>& get_excess_error_estimates() const;
   const bool contains_variance_estimates() const;
   const bool contains_error_estimates() const;
   const size_t size() const;
@@ -39,6 +42,7 @@ private:
   std::vector<double> predictions;
   std::vector<double> variance_estimates;
   std::vector<double> error_estimates;
+  std::vector<double> excess_error_estimates;
 };
 
 
