@@ -26,13 +26,14 @@
 
 class DefaultPredictionCollector: public PredictionCollector {
 public:
-  DefaultPredictionCollector(std::shared_ptr<DefaultPredictionStrategy> strategy,
-                              uint ci_group_size);
+  DefaultPredictionCollector(std::shared_ptr<DefaultPredictionStrategy> strategy);
 
   std::vector<Prediction> collect_predictions(const Forest& forest,
-                                              Data* prediction_data,
+                                              Data* train_data,
+                                              Data* data,
                                               const std::vector<std::vector<size_t>>& leaf_nodes_by_tree,
                                               const std::vector<std::vector<bool>>& valid_trees_by_sample,
+                                              bool estimate_variance,
                                               bool estimate_error);
 
 private:
@@ -40,7 +41,6 @@ private:
 
   std::shared_ptr<DefaultPredictionStrategy> strategy;
   SampleWeightComputer weight_computer;
-  uint ci_group_size;
 };
 
 
