@@ -24,13 +24,14 @@
 
 class OptimizedPredictionCollector: public PredictionCollector {
 public:
-  OptimizedPredictionCollector(std::shared_ptr<OptimizedPredictionStrategy> strategy,
-                               uint ci_group_size);
+  OptimizedPredictionCollector(std::shared_ptr<OptimizedPredictionStrategy> strategy);
 
   std::vector<Prediction> collect_predictions(const Forest& forest,
-                                              Data* prediction_data,
+                                              Data* train_data,
+                                              Data* data,
                                               const std::vector<std::vector<size_t>>& leaf_nodes_by_tree,
                                               const std::vector<std::vector<bool>>& valid_trees_by_sample,
+                                              bool estimate_variance,
                                               bool estimate_error);
 
 private:
@@ -44,7 +45,6 @@ private:
   void validate_prediction(size_t sample, Prediction prediction);
 
   std::shared_ptr<OptimizedPredictionStrategy> strategy;
-  uint ci_group_size;
 };
 
 
