@@ -23,9 +23,11 @@
 class PredictionCollector {
 public:
   virtual std::vector<Prediction> collect_predictions(const Forest& forest,
-                                                      Data* prediction_data,
+                                                      Data* train_data,
+                                                      Data* data,
                                                       const std::vector<std::vector<size_t>>& leaf_nodes_by_tree,
                                                       const std::vector<std::vector<bool>>& valid_trees_by_sample,
+                                                      bool estimate_variance,
                                                       bool estimate_error) = 0;
 };
 
