@@ -19,11 +19,11 @@
 
 std::unordered_map<size_t, double> NoopRelabelingStrategy::relabel(
     const std::vector<size_t>& samples,
-    const Observations& observations) {
+    const Data* data) {
 
   std::unordered_map<size_t, double> relabeled_observations;
   for (size_t sample : samples) {
-    double outcome = observations.get(Observations::OUTCOME, sample);
+    double outcome = data->get_outcome(sample);
     relabeled_observations[sample] = outcome;
   }
   return relabeled_observations;
