@@ -24,7 +24,7 @@ class NoopRelabelingStrategy: public RelabelingStrategy {
 public:
   std::unordered_map<size_t, double> relabel(
       const std::vector<size_t>& samples,
-      const Observations& observations);
+      const Data* data);
 };
 
 
