@@ -18,7 +18,7 @@
 #ifndef GRF_QUANTILERELABELINGSTRATEGY_H
 #define GRF_QUANTILERELABELINGSTRATEGY_H
 
-#include "commons/Observations.h"
+#include "commons/Data.h"
 #include "tree/Tree.h"
 #include "relabeling/RelabelingStrategy.h"
 
@@ -27,7 +27,7 @@ public:
   QuantileRelabelingStrategy(const std::vector<double>& quantiles);
   std::unordered_map<size_t, double> relabel(
       const std::vector<size_t>& samples,
-      const Observations& observations);
+      const Data* data);
 private:
   std::vector<double> quantiles;
 };
