@@ -136,15 +136,17 @@ private:
                      size_t num_samples,
                      const std::vector<double>& weights);
 
-  /**
-   * Knuth's algorithm for sampling without replacement, faster for larger num_samples
-   * Idea from Knuth 1985, The Art of Computer Programming, Vol. 2, Sec. 3.4.2 Algorithm S
+
+
+    /**
+   * Fisher-Yates algorithm for sampling without replacement, faster for larger num_samples
+   * Idea from Knuth 1985, The Art of Computer Programming, Vol. 2, Sec. 3.4.2 Algorithm P
    * @param result Vector to add results to. Will not be cleaned before filling.
    * @param max Specifies the interval to draw from:  0 ... (max-1).
    * @param skip Values to skip
    * @param num_samples Number of samples to draw
    */
-  void draw_knuth(std::vector<size_t>& result,
+  void draw_fisher_yates(std::vector<size_t>& result,
                   size_t max,
                   const std::set<size_t>& skip,
                   size_t num_samples);
