@@ -33,8 +33,7 @@
 class InstrumentalSplittingRuleFactory: public SplittingRuleFactory {
 public:
   InstrumentalSplittingRuleFactory();
-  std::shared_ptr<SplittingRule> create(Data* data,
-                                        const Observations& observations,
+  std::shared_ptr<SplittingRule> create(const Data* data,
                                         const TreeOptions& options);
 private:
   DISALLOW_COPY_AND_ASSIGN(InstrumentalSplittingRuleFactory);
