@@ -23,8 +23,7 @@
 ProbabilitySplittingRuleFactory::ProbabilitySplittingRuleFactory(size_t num_classes):
     num_classes(num_classes) {}
 
-std::shared_ptr<SplittingRule> ProbabilitySplittingRuleFactory::create(Data* data,
-                                                                       const Observations& observations,
+std::shared_ptr<SplittingRule> ProbabilitySplittingRuleFactory::create(const Data* data,
                                                                        const TreeOptions& options) {
   return std::shared_ptr<SplittingRule>(new ProbabilitySplittingRule(
       data, num_classes, options.get_alpha(), options.get_imbalance_penalty()));
