@@ -20,8 +20,7 @@
 
 RegressionSplittingRuleFactory::RegressionSplittingRuleFactory() {}
 
-std::shared_ptr<SplittingRule> RegressionSplittingRuleFactory::create(Data* data,
-                                                                      const Observations& observations,
+std::shared_ptr<SplittingRule> RegressionSplittingRuleFactory::create(const Data* data,
                                                                       const TreeOptions& options) {
   return std::shared_ptr<SplittingRule>(new RegressionSplittingRule(
       data, options.get_alpha(), options.get_imbalance_penalty()));
