@@ -30,12 +30,14 @@ public:
   TreeOptions(uint mtry,
               uint min_node_size,
               bool honesty,
+              double honesty_fraction,
               double alpha,
               double imbalance_penalty);
 
   uint get_mtry() const;
   uint get_min_node_size() const;
   bool get_honesty() const;
+  double get_honesty_fraction() const;
 
   /**
    * The minimum fraction of samples that are allowed to be on either
@@ -53,7 +55,7 @@ private:
   uint mtry;
   uint min_node_size;
   bool honesty;
-
+  double honesty_fraction;
   double alpha;
   double imbalance_penalty;
 };
